function FocPlaneVar_Invar_reconstr_illustr(WSz,Dmin,Dmax,Step,P);

% Demo for course "Introduction to Digital Holography"
% to demonstrate reconstruction of Fresnel hologram with shift variant and
% shift-invariant DFrT
% WSz - window size for removing hologram local mean
% Dmin,Dmxm - range of distances, 
% Focussing parameters: Dmin:Step:Dmax
% P - parameter for P-law displaying 
% Recommended: FocPlaneVar_Invar_reconstr_illustr(1,-0.5,0.5,0.02,0.5);
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il), 
% 
% mat-files hologr1 and hologr2 must be in the Matlab path


m=menu('Select','Off axis hologram','On axis hologram');
switch m,
    case 1, load hologr1;
        WSz=1;
        Par=inputdlg({'Minimal distance Dmin=','Maximal distance Dmax=','Distance step Step=','P-th law display parameter P='},.......
        'Enter parameters',1,{'-0.5','0.5','0.02','0.5'});
        Dmin=str2double(Par(1));
        Dmax=str2double(Par(2));
        Step=str2double(Par(3));
        P=str2double(Par(4));
        Hologr=double(Hologr);
        if WSz~=1, Hologr=Hologr-mean__ksn(Hologr,WSz,WSz);end
    case 2,
        load hologr2;
        WSz=1;
        Par=inputdlg({'Minimal distance Dmin=','Maximal distance Dmax=','Distance step Step=','P-th law display parameter P='},.......
        'Enter parameters',1,{'-0.68','0.68','0.04','0.5'});
        Dmin=str2double(Par(1));
        Dmax=str2double(Par(2));
        Step=str2double(Par(3));
        P=str2double(Par(4));
        Hologr=hologr2(:,1:1024);
end
n=1+0.5*(m-1);
[SzX,SzY]=size(Hologr);
figure(1);colormap(copper);
AbsHologr=uquan256(abs(Hologr));
Nd=fix((Dmax-Dmin)/Step)+1;
% k=1;
for D=[Dmin:Step:Dmax,Dmax:-Step:Dmin],
   OUT1=fresnel(Hologr,n*D,0,0,sign(D));
   OUT1=uquan256(OUT1);
   OUT1(SzX/2+1,:)=255;OUT1(:,SzY/2+1)=255;OUT1(:,SzY-1:SzY)=255;
%  OUT2=fresnel(Hologr,D,-SzX*D/2,-SzX*D/2,sign(D));
   OUT2=fresnel(Hologr,n*D,-n*SzX*D/2,-n*SzX*D/2,sign(D));
   OUT2=uquan256(OUT2);
   OUT2(SzX/2+1,:)=255;OUT2(:,SzY/2+1)=255;OUT2(:,1:2)=255;
   imshow([OUT1,OUT2].^P,[]);colormap(gray(256));colormap(copper);
   title(sprintf(['Focal plane variant (left) and invariant (right) Fresnel hologram reconstruction; mu^2=',num2str(n*abs(D))]),'FontWeight','Bold');
   drawnow
   pause(0.5);
%    k=k+1;
end


%----------------------------------------------------------------------
function OUT=fresnel(INPUT,D,Wx,Wy,Sign);

% Partial Fresnel Transform
% D - focussing parameter
% Wx,Wy - shift parameters
% Sign=1 for FFT, -1 for IFFT
% For BIAS hologram recommended: D around 0.25, Wx=Wy=0
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il), Dec. 24, 2001
% Usage OUT=fresnel(INPUT,D);

[SzX,SzY]=size(INPUT);
k=(0:SzX-1)';
l=0:SzY-1;
FrX=exp(-i*pi*k*D.*(k*D+2*Wx)/SzX);
FrY=exp(-i*pi*l*D.*(l*D+2*Wy)/SzY);
FrXY=kron(FrX,FrY);

if Sign==1,
   OUT=fftshift(abs(fft2(INPUT.*FrXY)));
else 
   OUT=fftshift(abs(ifft2(INPUT.*FrXY)));
end
%---------------------------------------------------------------------
function OUTIMG=uquan256(INPIMG);

% Uniform quantization of a signal into a range [0-255];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTIMG=uquan256(INPIMG);

mn=min(min(INPIMG));
mx=max(max(INPIMG));
 
OUTIMG=round(255*(INPIMG-mn)/(mx-mn));
%---------------------------------------------------------------------

