function fourier_vers_conv_demo_CRC;
% function fourier_vers_conv_demo_CRC(Hologr,Zmin,deltaZ,Zmax,Lambda,CameraPitch,Sign,FlagMovie);

% Generating movie frames from hologram Hologr by Juan Campos ReIm512_apod.mat
% For demonstrating similarity and dissimilarity of the reconstruction algorithms
% fourier_vers_conv_demo(ReIm512_mn_mask,25,1,85,458,5.5,1);
% Zmin, Zmax - distance in mm 25-85
% Lambda - wavelength, in nm 458
% CameraPitch - camera pitch in mcm 5.5 or 11
% Sign is a sign of i (sqrt(-1))
% MaskingFlag =1, hologram masking is applied for Fourier method
% If FlagMovie=1, frames are recorded to make a movie;
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)

Lambda=458;
CameraPitch=11; 
Zmin=25;
Zmax=136;
deltaZ=0.5;
Sign=1;
load ReIm512_mn_mask;
Hologr=ReIm512_mn_mask;
[SzX,SzY]=size(Hologr);
k=1:4:SzX;
l=1:4:SzY;
iptsetpref('ImshowInitialMagnification','fit');
for Z=Zmin:deltaZ:Zmax,
    [Ampl_Four_nm,Phase,Mu2X,Mu2Y]=fresn_hologr_reconstr_pls_mns(Hologr,Lambda,Z,CameraPitch,1,Sign,0);
    [Ampl_Four_m,Phase,Mu2X,Mu2Y]=fresn_hologr_reconstr_pls_mns(Hologr,Lambda,Z,CameraPitch,1,Sign,1);
    [Ampl_Conv,Phase,Mu2X,Mu2Y]=fresn_hologr_reconstr_pls_mns(Hologr,Lambda,Z,CameraPitch,2,Sign);  
    Ampl_Four_nm=Ampl_Four_nm(k,l)/max(Ampl_Four_nm(:));
    Ampl_Four_m=Ampl_Four_m(k,l)/max(Ampl_Four_m(:));
    Ampl_Conv=Ampl_Conv(k,l)/max(Ampl_Conv(:));
    [SZx,SZy]=size(Ampl_Four_nm);
    imshow([Ampl_Four_nm,zeros(SZx,10),Ampl_Four_m,zeros(SZx,10),Ampl_Conv],[]);colormap(copper)
    title(['Reconstruction, from left to right: Fourier method; Fourier method & hologram cropping;' ...........
        ' Convolution method.\newline Z=',num2str(Z),'mm; \mu^2=',num2str(Mu2X,3)],'FontWeight','Bold');
    drawnow
end

%-----------------------------------------------------------
function [Ampl,Phase,Mu2X,Mu2Y]=fresn_hologr_reconstr_pls_mns(Hol,Lambda,Z,CameraPitch,method,Sign,MaskingFlag);

% Reconstruction of Fresnel holograms: Fourier and Convolution algorithm
% Hol - hologram array
% Lambda - wave length (nanometers);
% Z - object-to-camera distance (millimeters);
% method = 1 for Fourier reconstruction algorithm and 2 for Convolution algorithm
% Sign is a sign of "i" in all algorithms
% CameraPitch - camera sensor's pitch (micrometers);
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il),
%  
% MaskingFlag = 1, if for Fourier algorithm hologram masking is to be applied

Lambda=Lambda*10^(-9);
Z=Z*10^(-3);
CameraPitch=CameraPitch*10^(-6);
[SzX,SzY]=size(Hol);

Mu2X=Lambda*Z/(SzX*CameraPitch^2);
MuX=sqrt(Mu2X);
Mu2Y=Lambda*Z/(SzY*CameraPitch^2);
MuY=sqrt(Mu2Y);

wX=MuX*(SzX-1)/2;
wY=MuY*(SzY-1)/2;

k=(0:SzX-1)';
l=0:SzY-1;
r=(0:SzX-1)';
s=0:SzY-1;

switch method
case 1         %Fourier reconstruction algorithm
    
    Dx=1/MuX;Dy=1/MuY;
    Wx=-SzX*Dx/2;
    Wy=-SzX*Dy/2;
    
    if MaskingFlag==1,
        Mmin=fix(SzX*(1-Mu2X)/2)+1;if Mmin<1, Mmin=1;end;
        Mmax=fix(SzX*(1+Mu2X)/2);  if Mmax>SzX,Mmax=SzX;end
        Nmin=fix(SzY*(1-Mu2Y)/2)+1;if Nmin<1,Nmin=1;end;
        Nmax=fix(SzY*(1+Mu2Y)/2);  if Nmax>SzY,Nmax=SzY;end
        m=Mmin:Mmax;
        n=Nmin:Nmax;
        Mask=zeros(SzX,SzY);
        Mask(m,n)=1;
        Hol=Hol.*Mask;
    end
    
%     FrX=exp(-i*pi*k*Dx.*(k*Dx+2*Wx)/SzX);
%     FrY=exp(-i*pi*l*Dy.*(l*Dy+2*Wy)/SzY);
    
%     FrX=exp(i*pi*k*Dx.*(k*Dx+2*Wx)/SzX);
%     FrY=exp(i*pi*l*Dy.*(l*Dy+2*Wy)/SzY);
%     FrXY=kron(FrX,FrY);
    
    
    if Sign==1,
        FrX=exp(i*pi*k*Dx.*(k*Dx+2*Wx)/SzX);
        FrY=exp(i*pi*l*Dy.*(l*Dy+2*Wy)/SzY);
        FrXY=kron(FrX,FrY);
        OUT=fftshift(fft2(Hol.*FrXY));
    else
        FrX=exp(-i*pi*k*Dx.*(k*Dx+2*Wx)/SzX);
        FrY=exp(-i*pi*l*Dy.*(l*Dy+2*Wy)/SzY);
        FrXY=kron(FrX,FrY);
        OUT=fftshift(ifft2(Hol.*FrXY));
    end

    
case 2         % Convolution reconstruction algorithm: Eq. 4.29a of the paper
    
    n=0:SzX-1;
    m=0:SzY-1;
    if Sign==-1,
        C_r=exp(-i*2*pi*r*(SzX-1)/(2*SzX));
        C_s=exp(-i*2*pi*s*(SzY-1)/(2*SzX));
%     
        C_rxs=kron(C_r,C_s);
        Hol_Four=ifft2(Hol.*C_rxs);
%   
        Chirp_r=exp(i*pi*Mu2X*((r-(SzX-1)/2).^2)/SzX);
        Chirp_s=exp(i*pi*Mu2Y*((s-(SzY-1)/2).^2)/SzY);
        Chirp_rxs=kron(Chirp_r,Chirp_s);
        OUT=fft2(Hol_Four.*Chirp_rxs);
    else
        C_r=exp(i*2*pi*r*(SzX-1)/(2*SzX));
        C_s=exp(i*2*pi*s*(SzY-1)/(2*SzX));
%     
        C_rxs=kron(C_r,C_s);
        Hol_Four=fft2(Hol.*C_rxs);
%   
        Chirp_r=exp(-i*pi*Mu2X*((r-(SzX-1)/2).^2)/SzX);
        Chirp_s=exp(-i*pi*Mu2Y*((s-(SzY-1)/2).^2)/SzY);
        Chirp_rxs=kron(Chirp_r,Chirp_s);
        OUT=ifft2(Hol_Four.*Chirp_rxs);
    end
   
end
Ampl=abs(OUT);
Phase=angle(OUT);

    

